#ifndef __HESTON_MC_CPP
#define __HESTON_MC_CPP

#include "heston_mc.h"

// HestonEuler
// ===========

HestonEuler::HestonEuler(Option* _pOption,
                         DCO_T<double> _kappa, DCO_T<double> _theta,
                         DCO_T<double> _xi, DCO_T<double> _rho) :
  pOption(_pOption), kappa(_kappa), theta(_theta), xi(_xi), rho(_rho) {}

HestonEuler::~HestonEuler() {}

void HestonEuler::calc_vol_path(const std::vector<DCO_T<double>>& vol_draws, 
                                std::vector<DCO_T<double>>& vol_path) { 
  using namespace std; // for max
  size_t vec_size = vol_draws.size();
  DCO_T<double> dt = pOption->T/static_cast<DCO_T<double>>(vec_size);

  // Iterate through the correlated random draws vector and
  // use the 'Full Truncation' scheme to create the volatility path
  for (int i=1; i<vec_size; i++) {
    DCO_T<double> v_max = max(vol_path[i-1], 0.0);
    DCO_T<double> d_sqrt=sqrt(v_max*dt);
    if (dco::value(v_max)==0) // instance of v_max is passive
      dco::derivative(d_sqrt)=0;
    vol_path[i] = vol_path[i-1] + kappa * dt * (theta - v_max) +
      xi * d_sqrt * vol_draws[i-1];
  }
}

void HestonEuler::calc_spot_path(const std::vector<DCO_T<double>>& spot_draws, 
                                 const std::vector<DCO_T<double>>& vol_path, 
                                 std::vector<DCO_T<double>>& spot_path) {
  using namespace std; // for max
  size_t vec_size = spot_draws.size();
  DCO_T<double> dt = pOption->T/static_cast<DCO_T<double>>(vec_size);

  // Create the spot price path making use of the volatility
  // path. Uses a similar Euler Truncation method to the vol path.
  for (int i=1; i<vec_size; i++) {
    DCO_T<double> v_max = max(vol_path[i-1], 0.0);
     DCO_T<double> d_sqrt=sqrt(v_max*dt);
    if (dco::value(v_max)==0) // instance of v_max is passive
      dco::derivative(d_sqrt)=0;
    spot_path[i] = spot_path[i-1] * exp( (pOption->r - 0.5*v_max)*dt + 
        d_sqrt*spot_draws[i-1]);
  }
}

#endif
