#ifndef __STATISTICS_H
#define __STATISTICS_H

#include "DCO_T.h"

#include <cmath>
#include <vector>

class StatisticalDistribution {
 public:
  StatisticalDistribution();
  virtual ~StatisticalDistribution();

  // Distribution functions
  virtual DCO_T<double> pdf(const DCO_T<double>& x) const = 0;
  virtual DCO_T<double> cdf(const DCO_T<double>& x) const = 0;

  // Inverse cumulative distribution functions (aka the quantile function)
  virtual DCO_T<double> inv_cdf(const DCO_T<double>& quantile) const = 0;
  
  // Descriptive stats
  virtual DCO_T<double> mean() const = 0;
  virtual DCO_T<double> var() const = 0;
  virtual DCO_T<double> stdev() const = 0;

  // Obtain a sequence of random draws from this distribution
  virtual void random_draws(const std::vector<DCO_T<double>>& uniform_draws,
                            std::vector<DCO_T<double>>& dist_draws) = 0;
};

class StandardNormalDistribution : public StatisticalDistribution {
 public:
  StandardNormalDistribution();
  virtual ~StandardNormalDistribution();

  // Distribution functions
  virtual DCO_T<double> pdf(const DCO_T<double>& x) const;
  virtual DCO_T<double> cdf(const DCO_T<double>& x) const;

  // Inverse cumulative distribution function (aka the probit function)
  virtual DCO_T<double> inv_cdf(const DCO_T<double>& quantile) const;

  // Descriptive stats
  virtual DCO_T<double> mean() const;   // equal to 0
  virtual DCO_T<double> var() const;    // equal to 1
  virtual DCO_T<double> stdev() const;  // equal to 1

  // Obtain a sequence of random draws from the standard normal distribution
  virtual void random_draws(const std::vector<DCO_T<double>>& uniform_draws,
                            std::vector<DCO_T<double>>& dist_draws);
};

#endif
