// Uwe.Naumann@nag.com

#include "f.h"

#include "my_dco.h" 

int main(int argc, char* argv[]) {
  assert(argc==3); 
  int np=stoi(argv[1]), ns=stoi(argv[2]); 

  using dco_partial_type=float; // partial derivative type
  using dco_adjoint_type=float; // adjoint type
  using dco_mode=dco::ga1s<dco_value_type,dco_partial_type,dco_adjoint_type>;
  using dco_type=dco_mode::type;

  dco_type x=1.;
  vector<dco_type> p(2*ns,1);
  dco::smart_tape_ptr_t<dco_mode> tape;

  tape->register_variable(x);
  for (int i=0;i<2*ns;i++) tape->register_variable(p[i]); 

  dco_type y=f(np,ns,x,p); 

  tape->register_output_variable(y); 
  dco::derivative(y)=1; 
  tape->interpret_adjoint(); 

  cout.precision(numeric_limits<dco_value_type>::digits10); 
  cout << "dy/dx=" << dco::derivative(x) << '\n';
  for (int i=0;i<2*ns;i++) 
    cout << "dy/dp[" << i << "]=" << dco::derivative(p[i]) << endl;
  cout << "y=" << dco::value(y) << endl;

  return 0;
}
