// Uwe.Naumann@nag.com

#include "f.h" 

#include "my_dco.h" 

template<>
dco_type f(int np, int ns, dco_type x, const vector<dco_type>& p) {
  default_random_engine g; normal_distribution<float> d(0,1);
  dco_type y=0, x_in=x; 
  auto tape=dco::tape(x);
  auto pos=tape->get_position();
  for (int i=0;i<np;++i) {
    x=x_in;
    for (int j=0;j<ns;++j) 
      x+=exp(-p[j]*x)/ns+p[ns+j]*x*sqrt(1./ns)*d(g);
    dco::derivative(x)=1./np;
    tape->interpret_adjoint_and_reset_to(pos);
    y+=x;
  }
  return y/np;
}

int main(int argc, char* argv[]) {
  assert(argc==3); 
  int np=stoi(argv[1]), ns=stoi(argv[2]); 

  dco_type x=1.; 
  vector<dco_type> p(2*ns,1);
  dco::smart_tape_ptr_t<dco_mode> tape; 

  tape->register_variable(x); 
  for (int i=0;i<2*ns;i++) tape->register_variable(p[i]); 

  dco_type y=f(np,ns,x,p); 

  cout.precision(numeric_limits<dco_value_type>::digits10); 
  cout << "dy/dx=" << dco::derivative(x) << '\n';
  for (int i=0;i<2*ns;i++) 
    cout << "dy/dp[" << i << "]=" << dco::derivative(p[i]) << endl;
  cout << "y=" << dco::value(y) << endl;

  return 0;
}
