// Uwe.Naumann@nag.com

#include "f.h"

#include "dco.hpp" 

int main(int argc, char* argv[]) {
  using namespace std;
  assert(argc==3); 
  int np=stoi(argv[1]), ns=stoi(argv[2]); 

  using dco_value_type=double; 
  const int VECTOR_SIZE=1;
  using dco_mode=dco::ga1v<dco_value_type,VECTOR_SIZE>; 
  using dco_type=dco_mode::type; 

  dco_type x=1.; 
  vector<dco_type> p(2*ns,1);
  dco::smart_tape_ptr_t<dco_mode> tape; 

  tape->register_variable(x); 
  for (int i=0;i<2*ns;i++) tape->register_variable(p[i]); 

  dco_type y=f(np,ns,x,p); 

  tape->register_output_variable(y); 
  dco::derivative(y)[0]=1; 
  tape->interpret_adjoint(); 

  cout.precision(numeric_limits<dco_value_type>::digits10); 
  cout << "dy/dx=" << dco::derivative(x)[0] << '\n';
  for (int i=0;i<2*ns;i++) 
    cout << "dy/dp[" << i << "]=" << dco::derivative(p[i])[0] << '\n';
  cout << "y=" << dco::value(y) << endl;

  return 0;
}
